@@ -197,23 +197,6 @@ def get_views(variables, node):
       seen.append(view.accessed_subset.items())
 
 
-def get_signatures(func):
-  """Gets the given function's signatures."""
-  if func.isinstance_PyTDFunction():
-    return [sig.signature for sig in func.signatures]
-  elif func.isinstance_InterpreterFunction():
-    return [f.signature for f in func.signature_functions()]
-  elif func.isinstance_BoundFunction():
-    sigs = get_signatures(func.underlying)
-    return [sig.drop_first_parameter() for sig in sigs]  # drop "self"
-  elif func.isinstance_ClassMethod() or func.isinstance_StaticMethod():
-    return get_signatures(func.method)
-  elif func.isinstance_SimpleFunction():
-    return [func.signature]
-  else:
-    raise NotImplementedError(func.__class__.__name__)
-
-
 def func_name_is_class_init(name):
   """Return True if |name| is that of a class' __init__ method."""
   # Python 3's MAKE_FUNCTION byte code takes an explicit fully qualified
@@ -847,7 +830,7 @@ def combine_substs(
     return ()
 
 
-def _flatten(value, classes):
+def flatten(value, classes):
   """Flatten the contents of value into classes.
 
   If value is a Class, it is appended to classes.
@@ -875,7 +858,7 @@ def _flatten(value, classes):
     ambiguous = False
     for var in value.pyval:
       if (len(var.bindings) != 1 or
-          _flatten(var.bindings[0].data, classes)):
+          flatten(var.bindings[0].data, classes)):
         # There were either multiple bindings or ambiguity deeper in the
         # recursion.
         ambiguous = True
@@ -899,7 +882,7 @@ def check_against_mro(vm, target, class_spec):
   """
   # Determine the flattened list of classes to check.
   classes = []
-  ambiguous = _flatten(class_spec, classes)
+  ambiguous = flatten(class_spec, classes)
 
   for c in classes:
     if vm.matcher(None).match_from_mro(target, c, allow_compat_builtins=False):
