@@ -211,8 +211,7 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
       d.from_annotation = name
     return node, value
 
-  def extract_and_init_annotation(self, node, name, var,
-                                  use_not_supported_yet=False):
+  def extract_and_init_annotation(self, node, name, var):
     """Extracts an annotation from var and instantiates it."""
     frame = self.vm.frame
     substs = frame.substs
@@ -232,8 +231,7 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
         itertools.chain(*substs, self.get_callable_type_parameter_names(var)))
     typ = self.extract_annotation(
         node, var, name, self.vm.simple_stack(),
-        allowed_type_params=allowed_type_params,
-        use_not_supported_yet=use_not_supported_yet)
+        allowed_type_params=allowed_type_params)
     if typ.formal:
       resolved_type = self.sub_one_annotation(node, typ, substs,
                                               instantiate_unbound=False)
@@ -251,17 +249,16 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
       return None, value
     annot = op.annotation
     frame = self.vm.frame
-    var, errorlog = abstract_utils.eval_expr(
-        self.vm, node, frame.f_globals, frame.f_locals, annot)
+    with self.vm.generate_late_annotations(self.vm.simple_stack()):
+      var, errorlog = abstract_utils.eval_expr(
+          self.vm, node, frame.f_globals, frame.f_locals, annot)
     if errorlog:
       self.vm.errorlog.invalid_annotation(
           self.vm.frames, annot, details=errorlog.details)
-    return self.extract_and_init_annotation(node, name, var,
-                                            use_not_supported_yet=True)
+    return self.extract_and_init_annotation(node, name, var)
 
   def extract_annotation(
-      self, node, var, name, stack, allowed_type_params=None,
-      use_not_supported_yet=True):
+      self, node, var, name, stack, allowed_type_params=None):
     """Returns an annotation extracted from 'var'.
 
     Args:
@@ -271,9 +268,6 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
       stack: The frame stack.
       allowed_type_params: Type parameters that are allowed to appear in the
         annotation. 'None' means all are allowed.
-      use_not_supported_yet: Temporary parameter to help transition the error
-        class for reporting 'type parameter not in scope' errors from
-        [not-supported-yet] to [invalid-annotation].
     """
     try:
       typ = abstract_utils.get_atomic_value(var)
@@ -299,19 +293,9 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
             frame_name = method.name
           details += f" for {desc} {frame_name!r}"
         if "AnyStr" in illegal_params:
-          if self.vm.PY2:
-            str_type = "typing.Text"
-          else:
-            str_type = "Union[str, bytes]"
-          details += (
-              f"\nNote: For all string types, use {str_type}.")
-        if use_not_supported_yet:
-          # TODO(b/186896951): Switch this to an [invalid-annotation] error.
-          self.vm.errorlog.not_supported_yet(
-              stack, "using type parameter in variable annotation",
-              details=details)
-        else:
-          self.vm.errorlog.invalid_annotation(stack, typ, details, name)
+          str_type = "Union[str, bytes]"
+          details += (f"\nNote: For all string types, use {str_type}.")
+        self.vm.errorlog.invalid_annotation(stack, typ, details, name)
         return self.vm.convert.unsolvable
     return typ
 
@@ -379,9 +363,8 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
         # Immediately try to evaluate the reference, generating LateAnnotation
         # objects as needed. We don't store the entire string as a
         # LateAnnotation because:
-        # - Starting in 3.8, or in 3.7 with __future__.annotations, all
-        #   annotations look like forward references - most of them don't need
-        #   to be late evaluated.
+        # - With __future__.annotations, all annotations look like forward
+        #   references - most of them don't need to be late evaluated.
         # - Given an expression like "Union[str, NotYetDefined]", we want to
         #   evaluate the union immediately so we don't end up with a complex
         #   LateAnnotation, which can lead to bugs when instantiated.
