@@ -14,8 +14,10 @@ from pytype import mixin
 from pytype import output
 from pytype import special_builtins
 from pytype import utils
+from pytype.overlays import attr_overlay
 from pytype.overlays import typing_overlay
 from pytype.pyc import loadmarshal
+from pytype.pyi import metadata
 from pytype.pytd import mro
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
@@ -267,6 +269,10 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     """Create an empty VM dict."""
     return abstract.Dict(self.vm).to_variable(node)
 
+  def build_map_class(self, node, type_params):
+    assert set(type_params) == {abstract_utils.K, abstract_utils.V}
+    return abstract.ParameterizedClass(self.dict_type, type_params, self.vm)
+
   def build_tuple(self, node, content):
     """Create a VM tuple from the given sequence."""
     return self.tuple_to_value(content).to_variable(node)
@@ -814,7 +820,21 @@ class Converter(utils.VirtualMachineWeakrefMixin):
           self._get_literal_value(pyval.value), subst, self.vm.root_node)
       return abstract.LiteralClass(value, self.vm)
     elif isinstance(pyval, pytd.Annotated):
-      return self.constant_to_value(pyval.base_type, subst, self.vm.root_node)
+      typ = self.constant_to_value(pyval.base_type, subst, self.vm.root_node)
+      if pyval.annotations[0] == "'pytype_metadata'":
+        try:
+          md = metadata.from_string(pyval.annotations[1])
+          if md["tag"] == "attr.ib":
+            ret = attr_overlay.AttribInstance.from_metadata(
+                self.vm, self.vm.root_node, typ, md)
+            return ret
+        except (IndexError, ValueError, TypeError, KeyError):
+          details = "Wrong format for pytype_metadata."
+          self.vm.errorlog.invalid_annotation(
+              self.vm.frames, pyval.annotations[1], details)
+          return typ
+      else:
+        return typ
     elif pyval.__class__ is tuple:  # only match raw tuple, not namedtuple/Node
       return self.tuple_to_value([
           self.constant_to_var(item, subst, self.vm.root_node)
