@@ -18,6 +18,31 @@ def argname(i):
   return "_" + str(i)
 
 
+def get_signatures(func):
+  """Gets the given function's signatures."""
+  if func.isinstance_PyTDFunction():
+    return [sig.signature for sig in func.signatures]
+  elif func.isinstance_InterpreterFunction():
+    return [f.signature for f in func.signature_functions()]
+  elif func.isinstance_BoundFunction():
+    sigs = get_signatures(func.underlying)
+    return [sig.drop_first_parameter() for sig in sigs]  # drop "self"
+  elif func.isinstance_ClassMethod() or func.isinstance_StaticMethod():
+    return get_signatures(func.method)
+  elif func.isinstance_SimpleFunction():
+    return [func.signature]
+  elif func.cls and func.cls.isinstance_CallableClass():
+    return [Signature.from_callable(func.cls)]
+  else:
+    if func.isinstance_Instance():
+      _, call_var = func.vm.attribute_handler.get_attribute(
+          func.vm.root_node, func, "__call__",
+          func.to_binding(func.vm.root_node))
+      if call_var and len(call_var.data) == 1:
+        return get_signatures(call_var.data[0])
+    raise NotImplementedError(func.__class__.__name__)
+
+
 def _print(t):
   return pytd_utils.Print(t.get_instance_type())
 
@@ -337,12 +362,15 @@ class Args(collections.namedtuple(
                  for var in args)
 
   def starstarargs_as_dict(self):
-    try:
-      args = self.starstarargs and abstract_utils.get_atomic_python_constant(
-          self.starstarargs, dict)
-    except abstract_utils.ConversionError:
-      args = None
-    return args
+    """Return **args as a python dict."""
+    # NOTE: We can't use get_atomic_python_constant here because starstarargs
+    # could have could_contain_anything set.
+    if not self.starstarargs or len(self.starstarargs.data) != 1:
+      return None
+    kwdict, = self.starstarargs.data
+    if not kwdict.isinstance_Dict():
+      return None
+    return kwdict.pyval
 
   def _expand_typed_star(self, vm, node, star, count):
     """Convert *xs: Sequence[T] -> [T, T, ...]."""
@@ -449,7 +477,26 @@ class Args(collections.namedtuple(
         namedargs = starstarargs_as_dict
       else:
         namedargs.update(node, starstarargs_as_dict)
-      starstarargs = None
+
+      # We have pulled out all the named args from the function call, so we need
+      # to delete them from starstarargs. If the original call contained
+      # **kwargs, starstarargs will have could_contain_anything set to True, so
+      # preserve it as an abstract dict. If not, we just had named args packed
+      # into starstarargs, so set starstarargs to None.
+      kwdict = starstarargs.data[0]
+      if kwdict.isinstance_Dict() and kwdict.could_contain_anything:
+        cls = kwdict.cls
+        if cls.isinstance_PyTDClass():
+          # If cls is not already parameterized with the key and value types, we
+          # parameterize it now to preserve them.
+          params = {
+              name: vm.convert.merge_classes(kwdict.get_instance_type_parameter(
+                  name, node).data)
+              for name in (abstract_utils.K, abstract_utils.V)}
+          cls = vm.convert.build_map_class(node, params)
+        starstarargs = cls.instantiate(node)
+      else:
+        starstarargs = None
     starargs_as_tuple = self.starargs_as_tuple(node, vm)
     if starargs_as_tuple is not None:
       if match_signature:
