@@ -3,10 +3,10 @@
 import collections
 import logging
 import os
+import pickle
 
 from typing import Dict, Iterable, Optional, Tuple
 
-from pytype import file_utils
 from pytype import module_utils
 from pytype import utils
 from pytype.pyi import parser
@@ -16,7 +16,6 @@ from pytype.pytd import pytd_utils
 from pytype.pytd import serialize_ast
 from pytype.pytd import typeshed
 from pytype.pytd import visitors
-from six.moves import cPickle
 
 log = logging.getLogger(__name__)
 
@@ -97,6 +96,7 @@ class Module:
     has_unresolved_pointers: Whether all ClassType pointers have been filled in
   """
 
+  # pylint: disable=redefined-outer-name
   def __init__(self, module_name, filename, ast,
                pickle=None, has_unresolved_pointers=True):
     self.module_name = module_name
@@ -104,6 +104,7 @@ class Module:
     self.ast = ast
     self.pickle = pickle
     self.has_unresolved_pointers = has_unresolved_pointers
+  # pylint: enable=redefined-outer-name
 
   def needs_unpickling(self):
     return bool(self.pickle)
@@ -130,8 +131,7 @@ class _ModuleMap:
 
   def __init__(self, python_version, modules=None):
     self.python_version = python_version
-    self._modules: Dict[str, Module] = modules or self._base_modules(
-        self.python_version)
+    self._modules: Dict[str, Module] = modules or self._base_modules()
     if self._modules["builtins"].needs_unpickling():
       self._unpickle_module(self._modules["builtins"])
     if self._modules["typing"].needs_unpickling():
@@ -185,8 +185,8 @@ class _ModuleMap:
             mod.module_name, mod.filename, mod.ast)
     return resolved_modules
 
-  def _base_modules(self, python_version):
-    bltins, typing = builtins.GetBuiltinsAndTyping(python_version)
+  def _base_modules(self):
+    bltins, typing = builtins.GetBuiltinsAndTyping()
     return {
         "builtins":
         Module("builtins", self.PREFIX + "builtins", bltins,
@@ -211,7 +211,7 @@ class _ModuleMap:
         seen.add(m)
       if not m.pickle:
         continue
-      loaded_ast = cPickle.loads(m.pickle)
+      loaded_ast = pickle.loads(m.pickle)
       deps = [d for d, _ in loaded_ast.dependencies if d != loaded_ast.ast.name]
       loaded_ast = serialize_ast.EnsureAstName(loaded_ast, m.module_name)
       assert m.module_name in self._modules
@@ -383,12 +383,18 @@ class _BuiltinLoader:
     assert ast.name == module
     return ast
 
-  def get_builtin(self, subdir, module_name):
-    builtin_dir = file_utils.get_versioned_path(subdir, self.python_version)
+  def get_builtin(self, builtin_dir, module_name):
+    """Load a stub that ships with pytype."""
     mod = self._parse_predefined(builtin_dir, module_name)
-    if not mod:
+    # For stubs in pytype's stubs/ directory, we use the module name prefixed
+    # with "pytd:" for the filename. Package filenames need an "/__init__.pyi"
+    # suffix for Module.is_package to recognize them.
+    if mod:
+      filename = module_name
+    else:
       mod = self._parse_predefined(builtin_dir, module_name, as_package=True)
-    return mod
+      filename = os.path.join(module_name, "__init__.pyi")
+    return filename, mod
 
 
 class Loader:
@@ -463,7 +469,7 @@ class Loader:
         for name, module in sorted(self._modules.items()))
     # Preparing an ast for pickling clears its class pointers, making it
     # unsuitable for reuse, so we have to discard the builtins cache.
-    builtins.InvalidateCache(self.python_version)
+    builtins.InvalidateCache()
     # Now pickle the pickles. We keep the "inner" modules as pickles as a
     # performance optimization - unpickling is slow.
     pytd_utils.SavePickle(
@@ -695,9 +701,9 @@ class Loader:
     """Load a pytd/pyi that ships with pytype or typeshed."""
     # Try our own type definitions first.
     if not third_party_only:
-      mod_ast = self._builtin_loader.get_builtin(subdir, module_name)
+      filename, mod_ast = self._builtin_loader.get_builtin(subdir, module_name)
       if mod_ast:
-        return self.load_file(filename=self.PREFIX + module_name,
+        return self.load_file(filename=self.PREFIX + filename,
                               module_name=module_name, mod_ast=mod_ast)
     if self.use_typeshed:
       return self._load_typeshed_builtin(subdir, module_name)
@@ -792,15 +798,6 @@ class Loader:
   def concat_all(self):
     return self._modules.concat_all()
 
-  def can_see(self, module):
-    """Reports whether the Loader is allowed to use the module."""
-    # If there is no imports_map or we are allowed to look up modules in
-    # typeshed, then any module that can be found can be used.
-    if self.imports_map is None or self.use_typeshed:
-      return True
-    return (module in self.imports_map or
-            f"{module}/__init__" in self.imports_map)
-
   def get_resolved_modules(self):
     """Gets a name -> ResolvedModule map of the loader's resolved modules."""
     return self._modules.get_resolved_modules()
