@@ -9,10 +9,13 @@ from typing import cast
 from pytype import abstract
 from pytype import abstract_utils
 from pytype import class_mixin
+from pytype import function
 from pytype import special_builtins
 from pytype import utils
+from pytype.overlays import attr_overlay
 from pytype.overlays import dataclass_overlay
 from pytype.overlays import typing_overlay
+from pytype.pyi import metadata
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import visitors
@@ -240,9 +243,13 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       return pytd_utils.JoinTypes(
           self.value_to_pytd_type(node, d, seen, view)
           for d in v.default.data)
+    elif isinstance(v, attr_overlay.AttribInstance):
+      ret = self.value_to_pytd_type(node, v.typ, seen, view)
+      md = metadata.to_pytd(v.to_metadata())
+      return pytd.Annotated(ret, ("'pytype_metadata'", md))
     elif isinstance(v, abstract.FUNCTION_TYPES):
       try:
-        signatures = abstract_utils.get_signatures(v)
+        signatures = function.get_signatures(v)
       except NotImplementedError:
         return pytd.NamedType("typing.Callable")
       if len(signatures) == 1:
@@ -387,9 +394,9 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     else:
       raise NotImplementedError(v.__class__.__name__)
 
-  def _ordered_attrs_to_instance_types(self, node, metadata, annots):
+  def _ordered_attrs_to_instance_types(self, node, attr_metadata, annots):
     """Get instance types for ordered attrs in the metadata."""
-    attrs = metadata.get("attr_order", [])
+    attrs = attr_metadata.get("attr_order", [])
     if not annots or not attrs:
       return
 
@@ -688,6 +695,11 @@ class Converter(utils.VirtualMachineWeakrefMixin):
             # type parameter; we do not want to merge in substituted values of
             # the type parameter.
             canonical_attributes.add(name)
+          if v.is_enum:
+            # If the containing class (v) is an enum, then output the instance
+            # attributes as properties.
+            # https://typing.readthedocs.io/en/latest/stubs.html#enums
+            typ = pytd.Annotated(typ, ("'property'",))
           constants[name].add_type(typ)
 
     for instance in v.canonical_instances:
