@@ -1,6 +1,7 @@
 """Printer to output pytd trees in pyi format."""
 
 import collections
+import copy
 import logging
 import re
 
@@ -32,6 +33,9 @@ class PrintVisitor(base_visitor.Visitor):
     self._typing_import_counts = collections.defaultdict(int)
     self.multiline_args = multiline_args
 
+  def Print(self, node):
+    return node.Visit(copy.deepcopy(self))
+
   def _IsEmptyTuple(self, t):
     """Check if it is an empty tuple."""
     assert isinstance(t, pytd.GenericType)
@@ -78,12 +82,12 @@ class PrintVisitor(base_visitor.Visitor):
         if not need_typing:
           names.discard(None)
       if None in names:
-        ret.append("import %s" % module)
+        ret.append(f"import {module}")
         names.remove(None)
 
       if names:
         name_str = ", ".join(sorted(names))
-        ret.append("from %s import %s" % (module, name_str))
+        ret.append(f"from {module} import {name_str}")
 
     return ret
 
@@ -93,12 +97,11 @@ class PrintVisitor(base_visitor.Visitor):
   def _FormatTypeParams(self, type_params):
     formatted_type_params = []
     for t in type_params:
-      args = ["'%s'" % t.name]
-      args += [c.Visit(PrintVisitor()) for c in t.constraints]
+      args = [f"'{t.name}'"]
+      args += [self.Print(c) for c in t.constraints]
       if t.bound:
-        args.append("bound=" + t.bound.Visit(PrintVisitor()))
-      formatted_type_params.append(
-          "%s = TypeVar(%s)" % (t.name, ", ".join(args)))
+        args.append(f"bound={self.Print(t.bound)}")
+      formatted_type_params.append(f"{t.name} = TypeVar({', '.join(args)})")
     return sorted(formatted_type_params)
 
   def _NameCollision(self, name):
@@ -112,7 +115,7 @@ class PrintVisitor(base_visitor.Visitor):
     self._typing_import_counts[name] += 1
     if self._NameCollision(name):
       self._RequireImport("typing")
-      return "typing." + name
+      return f"typing.{name}"
     else:
       self._RequireImport("typing", name)
       return name
@@ -164,7 +167,7 @@ class PrintVisitor(base_visitor.Visitor):
       assert name in ("True", "False"), name
       return name
     else:
-      return node.name + ": " + node.type
+      return f"{node.name}: {node.type}"
 
   def EnterAlias(self, _):
     self.old_imports = self.imports.copy()
@@ -180,21 +183,20 @@ class PrintVisitor(base_visitor.Visitor):
         if alias_name.startswith(f"{self._unit_name}."):
           alias_name = alias_name[len(self._unit_name)+1:]
         if name not in ("*", alias_name):
-          suffix += " as " + alias_name
+          suffix += f" as {alias_name}"
         self.imports = self.old_imports  # undo unnecessary imports change
-        return "from " + module + " import " + name + suffix
+        return f"from {module} import {name}{suffix}"
     elif isinstance(self.old_node.type, (pytd.Constant, pytd.Function)):
-      return self.old_node.type.Replace(name=node.name).Visit(PrintVisitor())
+      return self.Print(self.old_node.type.Replace(name=node.name))
     elif isinstance(self.old_node.type, pytd.Module):
       return node.type
-    return node.name + " = " + node.type
+    return f"{node.name} = {node.type}"
 
   def EnterClass(self, node):
     """Entering a class - record class name for children's use."""
     n = node.name
     if node.template:
-      n += "[{}]".format(
-          ", ".join(t.Visit(PrintVisitor()) for t in node.template))
+      n += "[{}]".format(", ".join(self.Print(t) for t in node.template))
     for member in node.methods + node.constants:
       self._class_members.add(member.name)
     self.class_names.append(n)
@@ -216,11 +218,11 @@ class PrintVisitor(base_visitor.Visitor):
       parents = ()
     if node.metaclass is not None:
       parents += ("metaclass=" + node.metaclass,)
-    parents_str = "(" + ", ".join(parents) + ")" if parents else ""
-    header = ["class " + node.name + parents_str + ":"]
+    parents_str = f"({', '.join(parents)})" if parents else ""
+    header = [f"class {node.name}{parents_str}:"]
     if node.slots is not None:
-      slots_str = ", ".join("\"%s\"" % s for s in node.slots)
-      slots = [self.INDENT + "__slots__ = [" + slots_str + "]"]
+      slots_str = ", ".join(f"\"{s}\"" for s in node.slots)
+      slots = [self.INDENT + f"__slots__ = [{slots_str}]"]
     else:
       slots = []
     decorators = ["@" + self.VisitNamedType(d)
@@ -276,11 +278,10 @@ class PrintVisitor(base_visitor.Visitor):
       # transformed **kwargs, so it was incremented at least once already.
       if isinstance(node.type.parameters[-1], pytd.AnythingType):
         self._typing_import_counts["Any"] -= 1
-      return node.Replace(type=node.type.parameters[-1], optional=False).Visit(
-          PrintVisitor())
+      return self.Print(
+          node.Replace(type=node.type.parameters[-1], optional=False))
     else:
-      return node.Replace(type=pytd.AnythingType(), optional=False).Visit(
-          PrintVisitor())
+      return self.Print(node.Replace(type=pytd.AnythingType(), optional=False))
 
   def VisitSignature(self, node):
     """Visit a signature, producing a string."""
@@ -289,7 +290,7 @@ class PrintVisitor(base_visitor.Visitor):
       self._FromTyping(return_type)
     else:
       return_type = node.return_type
-    ret = " -> " + return_type
+    ret = f" -> {return_type}"
 
     # Put parameters in the right order:
     # (arg1, arg2, *args, kwonly1, kwonly2, **kwargs)
@@ -307,10 +308,10 @@ class PrintVisitor(base_visitor.Visitor):
         break
     else:
       if starargs:
-        params += ("*" + starargs,)
+        params += (f"*{starargs}",)
     if self.old_node.starstarargs is not None:
       starstarargs = self._FormatContainerContents(self.old_node.starstarargs)
-      params += ("**" + starstarargs,)
+      params += (f"**{starstarargs}",)
 
     body = []
     # Handle Mutable parameters
@@ -321,8 +322,7 @@ class PrintVisitor(base_visitor.Visitor):
     # pylint: enable=no-member
     for name, new_type in mutable_params:
       body.append("\n{indent}{name} = {new_type}".format(
-          indent=self.INDENT, name=name,
-          new_type=new_type.Visit(PrintVisitor())))
+          indent=self.INDENT, name=name, new_type=self.Print(new_type)))
     for exc in node.exceptions:
       body.append("\n{indent}raise {exc}()".format(indent=self.INDENT, exc=exc))
     if not body:
@@ -428,9 +428,9 @@ class PrintVisitor(base_visitor.Visitor):
 
   def VisitModule(self, node):
     if node.is_aliased:
-      return "import %s as %s" % (node.module_name, node.name)
+      return f"import {node.module_name} as {node.name}"
     else:
-      return "import %s" % node.module_name
+      return f"import {node.module_name}"
 
   def MaybeCapitalize(self, name):
     """Capitalize a generic type, if necessary."""
@@ -455,8 +455,9 @@ class PrintVisitor(base_visitor.Visitor):
             "[" + ", ".join(parameters) + "]")
 
   def VisitCallableType(self, node):
-    return "%s[[%s], %s]" % (self.MaybeCapitalize(node.base_type),
-                             ", ".join(node.args), node.ret)
+    typ = self.MaybeCapitalize(node.base_type)
+    args = ", ".join(node.args)
+    return f"{typ}[[{args}], {node.ret}]"
 
   def VisitTupleType(self, node):
     return self.VisitGenericType(node)
@@ -539,9 +540,9 @@ class PrintVisitor(base_visitor.Visitor):
 
   def VisitLiteral(self, node):
     base = self._ImportTypingExtension("Literal")
-    return "%s[%s]" % (base, node.value)
+    return f"{base}[{node.value}]"
 
   def VisitAnnotated(self, node):
     base = self._ImportTypingExtension("Annotated")
     annotations = ", ".join(node.annotations)
-    return "%s[%s, %s]" % (base, node.base_type, annotations)
+    return f"{base}[{node.base_type}, {annotations}]"
