@@ -9,7 +9,6 @@ from pytype.pytd import serialize_ast
 from pytype.pytd import visitors
 from pytype.pytd.parse import parser_test_base
 
-import six
 import unittest
 
 
@@ -26,9 +25,9 @@ class TestUtils(parser_test_base.ParserTest):
     c1 = ast.Lookup("c1").type
     c2 = ast.Lookup("c2").type
     c3 = ast.Lookup("c3").type
-    six.assertCountEqual(self, pytd_utils.UnpackUnion(c1), c1.type_list)
-    six.assertCountEqual(self, pytd_utils.UnpackUnion(c2), [c2])
-    six.assertCountEqual(self, pytd_utils.UnpackUnion(c3), [c3])
+    self.assertCountEqual(pytd_utils.UnpackUnion(c1), c1.type_list)
+    self.assertCountEqual(pytd_utils.UnpackUnion(c2), [c2])
+    self.assertCountEqual(pytd_utils.UnpackUnion(c3), [c3])
 
   def test_concat(self):
     """Test for concatenating two pytd ASTs."""
@@ -37,7 +36,7 @@ class TestUtils(parser_test_base.ParserTest):
 
       def f1() -> int: ...
 
-      class Class1(object):
+      class Class1:
         pass
     """)
     ast2 = self.Parse("""
@@ -45,7 +44,7 @@ class TestUtils(parser_test_base.ParserTest):
 
       def f2() -> int: ...
 
-      class Class2(object):
+      class Class2:
         pass
     """)
     expected = textwrap.dedent("""
@@ -55,10 +54,10 @@ class TestUtils(parser_test_base.ParserTest):
       def f1() -> int: ...
       def f2() -> int: ...
 
-      class Class1(object):
+      class Class1:
           pass
 
-      class Class2(object):
+      class Class2:
           pass
     """)
     combined = pytd_utils.Concat(ast1, ast2)
@@ -166,14 +165,14 @@ class TestUtils(parser_test_base.ParserTest):
       c = ...  # type: int
       def f(x: int) -> int: ...
       def f(x: float) -> float: ...
-      class A(object):
+      class A:
         pass
     """)
     ast2 = self.Parse("""
       c = ...  # type: float
       d = ...  # type: int
       def f(x: complex) -> complex: ...
-      class B(object):
+      class B:
         pass
     """)
     w = pytd_utils.WrapTypeDeclUnit(
@@ -187,73 +186,13 @@ class TestUtils(parser_test_base.ParserTest):
       def f(x: int) -> int: ...
       def f(x: float) -> float: ...
       def f(x: complex) -> complex: ...
-      class A(object):
+      class A:
         pass
-      class B(object):
+      class B:
         pass
     """)
     self.AssertSourceEquals(w, expected)
 
-  def test_wraps_dict(self):
-    class A(pytd_utils.WrapsDict("m")):
-      pass
-    a = A()
-    a.m = {}
-    a.m = {"foo": 1, "bar": 2}
-    self.assertEqual(a.get("x", "baz"), "baz")
-    self.assertNotIn("x", a)
-    self.assertEqual(a.get("foo"), 1)
-    self.assertEqual(a["foo"], 1)
-    self.assertIn("foo", a)
-    self.assertIn("bar", a)
-    self.assertEqual(a.copy(), a.m)
-    six.assertCountEqual(self, iter(a), ["foo", "bar"])
-    six.assertCountEqual(self, a.keys(), ["foo", "bar"])
-    six.assertCountEqual(self, a.viewkeys(), ["foo", "bar"])
-    six.assertCountEqual(self, a.iterkeys(), ["foo", "bar"])
-    six.assertCountEqual(self, a.values(), [1, 2])
-    six.assertCountEqual(self, a.viewvalues(), [1, 2])
-    six.assertCountEqual(self, a.itervalues(), [1, 2])
-    six.assertCountEqual(self, a.items(), [("foo", 1), ("bar", 2)])
-    six.assertCountEqual(self, a.viewitems(), [("foo", 1), ("bar", 2)])
-    six.assertCountEqual(self, a.iteritems(), [("foo", 1), ("bar", 2)])
-    self.assertFalse(hasattr(a, "popitem"))
-
-  def test_wraps_writable_dict(self):
-    class A(pytd_utils.WrapsDict("m", writable=True)):
-      pass
-    a = A()
-    a.m = {}
-    a.m = {"foo": 1, "bar": 2}
-    self.assertIn("foo", a)
-    self.assertIn("bar", a)
-    del a["foo"]
-    a["bar"] = 3
-    self.assertNotIn("foo", a)
-    self.assertIn("bar", a)
-    value = a.pop("bar")
-    self.assertEqual(3, value)
-    self.assertNotIn("bar", a)
-    a["new"] = 7
-    item = a.popitem()
-    self.assertEqual(item, ("new", 7))
-    a["1"] = 1
-    a.setdefault("1", 11)
-    a.setdefault("2", 22)
-    self.assertEqual(a["1"], 1)
-    self.assertEqual(a["2"], 22)
-    a.update({"3": 33})
-    six.assertCountEqual(self, a.items(), (("1", 1), ("2", 22), ("3", 33)))
-    a.clear()
-    six.assertCountEqual(self, a.items(), ())
-
-  def test_wraps_dict_with_length(self):
-    class A(pytd_utils.WrapsDict("m", implement_len=True)):
-      pass
-    a = A()
-    a.m = {x: x for x in range(42)}
-    self.assertEqual(42, len(a))
-
   def test_builtin_alias(self):
     src = "Number = int"
     ast = parser.parse_string(src, python_version=self.python_version)
@@ -278,7 +217,7 @@ class TestUtils(parser_test_base.ParserTest):
 
       x = ...  # type: typing.List[str]
 
-      class MyClass(object):
+      class MyClass:
           List = ...  # type: Any
           x = ...  # type: typing.List[str]
     """), python_version=self.python_version)
@@ -431,43 +370,32 @@ class TestUtils(parser_test_base.ParserTest):
     self.assertEqual(normalize(pytd_utils.ASTdiff(tree1, tree1)), src1)
     self.assertEqual(normalize(pytd_utils.ASTdiff(tree2, tree2)), src2)
     diff_pattern = r"(?s)- b.*\+ b"
-    six.assertRegex(self, normalize(pytd_utils.ASTdiff(tree1, tree2)),
-                    diff_pattern)
-    six.assertRegex(self, normalize(pytd_utils.ASTdiff(tree2, tree1)),
-                    diff_pattern)
+    self.assertRegex(normalize(pytd_utils.ASTdiff(tree1, tree2)), diff_pattern)
+    self.assertRegex(normalize(pytd_utils.ASTdiff(tree2, tree1)), diff_pattern)
 
 
 class TestDataFiles(parser_test_base.ParserTest):
   """Test pytd_utils.GetPredefinedFile()."""
 
-  BUILTINS = "builtins/3"
+  BUILTINS = "builtins"
 
   def test_get_predefined_file_basic(self):
     # smoke test, only checks that it doesn't throw, the filepath is correct,
     # and the result is a string
     path, src = pytd_utils.GetPredefinedFile(self.BUILTINS, "builtins")
-    self.assertEqual(path, "stubs/builtins/3/builtins.pytd")
+    self.assertEqual(path, "stubs/builtins/builtins.pytd")
     self.assertIsInstance(src, str)
 
   def test_get_predefined_file_throws(self):
     # smoke test, only checks that it does throw
-    with six.assertRaisesRegex(
-        self, IOError,
+    with self.assertRaisesRegex(
+        IOError,
         r"File not found|Resource not found|No such file or directory"):
       pytd_utils.GetPredefinedFile(self.BUILTINS, "-this-file-does-not-exist")
 
-  def test_pytd_builtin2(self):
-    """Verify 'import sys' for python2."""
-    subdir = "builtins/2"
-    _, import_contents = pytd_utils.GetPredefinedFile(subdir, "builtins")
-    with open(os.path.join(os.path.dirname(file_utils.__file__), "stubs",
-                           subdir, "builtins.pytd"), "r") as fi:
-      file_contents = fi.read()
-    self.assertMultiLineEqual(import_contents, file_contents)
-
   def test_pytd_builtin3(self):
     """Verify 'import sys' for python3."""
-    subdir = "builtins/3"
+    subdir = "builtins"
     _, import_contents = pytd_utils.GetPredefinedFile(subdir, "builtins")
     with open(os.path.join(os.path.dirname(file_utils.__file__), "stubs",
                            subdir, "builtins.pytd"), "r") as fi:
@@ -475,9 +403,9 @@ class TestDataFiles(parser_test_base.ParserTest):
     self.assertMultiLineEqual(import_contents, file_contents)
 
   def test_pytd_builtin_is_package(self):
-    subdir = "builtins/2and3"
+    subdir = "builtins"
     path, _ = pytd_utils.GetPredefinedFile(subdir, "attr", as_package=True)
-    self.assertEqual(path, "stubs/builtins/2and3/attr/__init__.pytd")
+    self.assertEqual(path, "stubs/builtins/attr/__init__.pytd")
 
   def test_type_builder(self):
     t = pytd_utils.TypeBuilder()
@@ -538,6 +466,21 @@ class PrintTest(parser_test_base.ParserTest):
       x: Literal['x', 'y']
     """).strip())
 
+  def test_reuse_union_name(self):
+    src = """
+      import typing
+      from typing import Callable, Iterable, Tuple
+
+      class Node: ...
+
+      class Union:
+          _predicates: Tuple[Callable[[typing.Union[Iterable[Node], Node]], bool], ...]
+          def __init__(self, *predicates: Callable[[typing.Union[Iterable[Node], Node]], bool]) -> None: ...
+    """
+    ast = self.Parse(src)
+    self.assertMultiLineEqual(pytd_utils.Print(ast),
+                              textwrap.dedent(src).strip())
+
 
 if __name__ == "__main__":
   unittest.main()
