@@ -246,12 +246,7 @@ CMP_IS_NOT = 9
 CMP_EXC_MATCH = 10
 
 
-CMP_ALWAYS_SUPPORTED_PY2 = frozenset({
-    CMP_LT, CMP_LE, CMP_EQ, CMP_NE, CMP_GT, CMP_GE, CMP_IS, CMP_IS_NOT
-})
-
-
-CMP_ALWAYS_SUPPORTED_PY3 = frozenset({
+CMP_ALWAYS_SUPPORTED = frozenset({
     CMP_EQ, CMP_NE, CMP_IS, CMP_IS_NOT
 })
 
