@@ -79,8 +79,6 @@ _PYTYPE_SINGLE_ITEMS = {
     'protocols': Item(None, 'False', ArgInfo('--protocols', None), None),
     'strict_import': Item(
         None, 'False', ArgInfo('--strict-import', None), None),
-    'enforce_noniterable_strings': Item(
-        None, 'True', ArgInfo('--enforce-noniterable-strings', None), None),
 }
 
 
